@@ -1,10 +1,5 @@
 from pathlib import Path
 import numpy as np
-import tkinter as tk
-from tkinter import font  as tkfont
-from tkinter import filedialog
-from tkinter import ttk
-from tkinter import *
 import json
 import cv2
 import math
@@ -14,12 +9,20 @@ import json
 import os
 import subprocess
 import sys
+import pandas as pd
 globalVariables = getGlobalVariables()
 
+try:
+  from PyQt6.QtWidgets import QFileDialog
+except ImportError:
+  from PyQt5.QtWidgets import QFileDialog
+
 from zebrazoom.dataAnalysis.datasetcreation.createDataFrame import createDataFrame
 from zebrazoom.dataAnalysis.dataanalysis.populationComparaison import populationComparaison
 from zebrazoom.dataAnalysis.dataanalysis.applyClustering import applyClustering
 
+from zebrazoom.dataAnalysis.datasetcreation.generatePklDataFileForVideo import generatePklDataFileForVideo
+
 def openExperimentOrganizationExcelFolder(self, homeDirectory):
   dir_path = os.path.join(homeDirectory,'dataAnalysis/experimentOrganizationExcel/')
   if sys.platform == "win32":
@@ -27,7 +30,7 @@ def openExperimentOrganizationExcelFolder(self, homeDirectory):
   else:
     opener ="open" if sys.platform == "darwin" else "xdg-open"
     subprocess.call([opener, dir_path])
-    
+
 
 def openAnalysisFolder(self, homeDirectory, specificDirectory):
   dir_path = os.path.join(os.path.join(homeDirectory,'dataAnalysis'), specificDirectory)
@@ -39,32 +42,32 @@ def openAnalysisFolder(self, homeDirectory, specificDirectory):
 
 
 def chooseExperimentOrganizationExcel(self, controller):
-  
+
   cur_dir_path = os.path.dirname(os.path.realpath(__file__))
   cur_dir_path = Path(cur_dir_path)
   cur_dir_path = cur_dir_path.parent.parent
-  
-  if globalVariables["mac"]:
-    experimentOrganizationExcel = filedialog.askopenfilename(initialdir = os.path.join(cur_dir_path, 'dataAnalysis/experimentOrganizationExcel/'), title = "Select the excel file describing your experiments")
-  else:
-    experimentOrganizationExcel = filedialog.askopenfilename(initialdir = os.path.join(cur_dir_path, 'dataAnalysis/experimentOrganizationExcel/'), title = "Select the excel file describing your experiments",filetypes = (("video","*.*"),("all files","*.*")))
-  
+
+  experimentOrganizationExcel, _ =  QFileDialog.getOpenFileName(self.window, "Select the excel file describing your experiments", os.path.join(cur_dir_path, 'dataAnalysis/experimentOrganizationExcel/'), "All files(*)")
+
   array = os.path.split(experimentOrganizationExcel)
-  
+
   self.experimentOrganizationExcel = array[len(array)-1]
   self.experimentOrganizationExcelFileAndFolder = ''.join(array[0:len(array)-1])
-  
+
   controller.show_frame("ChooseDataAnalysisMethod")
 
 
-def populationComparison(self, controller, TailTrackingParameters=0, saveInMatlabFormat=0, saveRawData=0, minNbBendForBoutDetect=3, discard=0, keep=1, frameStepForDistanceCalculation=4):
-
+def populationComparison(self, controller, TailTrackingParameters=0, saveInMatlabFormat=0, saveRawData=0, minNbBendForBoutDetect=3, discard=0, keep=1, frameStepForDistanceCalculation='4'):
+  
+  if len(frameStepForDistanceCalculation) == 0:
+    frameStepForDistanceCalculation = '4'
+  
   if discard == 0 and keep == 0:
     keep = 1
-  
+
   if len(minNbBendForBoutDetect) == 0:
     minNbBendForBoutDetect = 3
-  
+
   cur_dir_path = os.path.dirname(os.path.realpath(__file__))
   cur_dir_path = Path(cur_dir_path)
   cur_dir_path = cur_dir_path.parent.parent
@@ -94,40 +97,50 @@ def populationComparison(self, controller, TailTrackingParameters=0, saveInMatla
     'saveAllBoutsSuperStructuresInMatlabFormat' : saveRawData,
     'frameStepForDistanceCalculation'           : frameStepForDistanceCalculation
   }
-
-  [conditions, genotypes, nbFramesTakenIntoAccount, globParam] = createDataFrame(dataframeOptions)
-
+  
+  generatePklDataFileForVideo(os.path.join(self.experimentOrganizationExcelFileAndFolder, self.experimentOrganizationExcel), ZZoutputLocation, frameStepForDistanceCalculation)
+  
+  [conditions, genotypes, nbFramesTakenIntoAccount, globParam] = createDataFrame(dataframeOptions, "", 0, ['BoutFrameNumberStart', 'tailAngleSymmetry', 'secondBendAmpDividedByFirst', 'tailAngleIntegral'])
+  
   # Plotting for the different conditions
   nameOfFile = dataframeOptions['nameOfFile']
   resFolder  = dataframeOptions['resFolder']
-  
+
   # Mixing up all the bouts
   populationComparaison(nameOfFile, resFolder, globParam, conditions, genotypes, os.path.join(cur_dir_path, os.path.join('dataAnalysis', 'resultsKinematic')), 0, True)
-  
+
   populationComparaison(nameOfFile, resFolder, globParam, conditions, genotypes, os.path.join(cur_dir_path, os.path.join('dataAnalysis', 'resultsKinematic')), 0, False)
-  
+
   # First median per well for each kinematic parameter
   populationComparaison(nameOfFile, resFolder, globParam, conditions, genotypes, os.path.join(cur_dir_path, os.path.join('dataAnalysis', 'resultsKinematic')), 1, True)
-  
+
   populationComparaison(nameOfFile, resFolder, globParam, conditions, genotypes, os.path.join(cur_dir_path, os.path.join('dataAnalysis', 'resultsKinematic')), 1, False)
-  
+
   controller.show_frame("AnalysisOutputFolderPopulation")
 
 
-def boutClustering(self, controller, nbClustersToFind, FreelySwimming, HeadEmbeded, minNbBendForBoutDetect=3, nbVideosToSave=0):
+def boutClustering(self, controller, nbClustersToFind, FreelySwimming, HeadEmbeded, minNbBendForBoutDetect=3, nbVideosToSave=0, modelUsedForClustering=0, removeOutliers=False, frameStepForDistanceCalculation='4'):
+
+  if len(frameStepForDistanceCalculation) == 0:
+    frameStepForDistanceCalculation = '4'
   
+  if modelUsedForClustering == 0:
+    modelUsedForClustering = 'KMeans'
+  else:
+    modelUsedForClustering = 'GaussianMixture'
+
   if len(minNbBendForBoutDetect) == 0:
     minNbBendForBoutDetect = 3
   else:
     minNbBendForBoutDetect = int(minNbBendForBoutDetect)
-  
+
   if len(nbVideosToSave) == 0:
     nbVideosToSave = 0
   else:
     nbVideosToSave = int(nbVideosToSave)
-  
+
   videoSaveFirstTenBouts = True if nbVideosToSave else False
-  
+
   cur_dir_path = os.path.dirname(os.path.realpath(__file__))
   cur_dir_path = Path(cur_dir_path)
   cur_dir_path = cur_dir_path.parent.parent
@@ -156,7 +169,9 @@ def boutClustering(self, controller, nbClustersToFind, FreelySwimming, HeadEmbed
   if int(FreelySwimming):
     dataframeOptions['computeMassCenterParamForCluster'] = True
     
-  [conditions, genotypes, nbFramesTakenIntoAccount, globParam] = createDataFrame(dataframeOptions)
+  generatePklDataFileForVideo(os.path.join(self.experimentOrganizationExcelFileAndFolder, self.experimentOrganizationExcel), ZZoutputLocation, frameStepForDistanceCalculation)
+    
+  [conditions, genotypes, nbFramesTakenIntoAccount, globParam] = createDataFrame(dataframeOptions, "", 0, ['BoutFrameNumberStart', 'tailAngleSymmetry', 'secondBendAmpDividedByFirst', 'tailAngleIntegral'])
   # Applying the clustering on this dataframe
   clusteringOptions = {
     'analyzeAllWellsAtTheSameTime' : 0, # put this to 1 for head-embedded videos, and to 0 for multi-well videos
@@ -178,10 +193,14 @@ def boutClustering(self, controller, nbClustersToFind, FreelySwimming, HeadEmbed
     'globalParametersCalculations' : True,
     'nbVideosToSave' : nbVideosToSave,
     'resFolder' : os.path.join(os.path.join(cur_dir_path, 'dataAnalysis'),'data/'),
-    'nameOfFile' : self.experimentOrganizationExcel.split(".")[0]
+    'nameOfFile' : self.experimentOrganizationExcel.split(".")[0],
+    'modelUsedForClustering' : modelUsedForClustering,
+    'removeOutliers'         : removeOutliers
   }
   if int(FreelySwimming):
     clusteringOptions['useAnglesSpeedHeading'] = True
+    # clusteringOptions['useAngleAnd3GlobalParameters'] = True
+    # clusteringOptions['useFreqAmpAsym'] = True
   if int(HeadEmbeded):
     clusteringOptions['useAngles'] = True
   # Applies the clustering
