@@ -128,7 +128,7 @@ def applyClustering(clusteringOptions, classifier, outputFolder, ZZoutputLocatio
   # if modelUsedForClustering == 'KMeans':
   scaler = StandardScaler()
   allInstaValues = scaler.fit_transform(allInstaValues)
-
+  
   allInstaValuesLenBef = len(allInstaValues)
   try:
     dfParam = dfParam.drop([idx for idx, val in enumerate(~np.isnan(allInstaValues).any(axis=1)) if not(val)])
@@ -144,6 +144,9 @@ def applyClustering(clusteringOptions, classifier, outputFolder, ZZoutputLocatio
   else:
     print("all bouts were kept (no nan values)")
   
+  if 'level_0' in dfParam.columns:
+    dfParam = dfParam.drop(['level_0'], axis=1)
+  
   dfParam = dfParam.reset_index()
   
   if removeOutliers:
