@@ -17,7 +17,7 @@ from zebrazoom.dataAnalysis.datasetcreation.getTailAngleRecalculated2 import get
 from zebrazoom.dataAnalysis.datasetcreation.gatherInitialRawData import gatherInitialRawData
 import pickle
 
-def createDataFrame(dataframeOptions):
+def createDataFrame(dataframeOptions, excelFileDataFrame="", forcePandasDfRecreation=0, addToGlobalParameters=0):
 
   # Gathering user-inputed information about how to create the dataframe of parameters for the whole set of videos
   
@@ -49,7 +49,13 @@ def createDataFrame(dataframeOptions):
   
   # If nbFramesTakenIntoAccount was not specified, finds an appropriate value for it
   nbFramesTakenIntoAccount          = dataframeOptions['nbFramesTakenIntoAccount']
-  excelFile = pd.read_excel(os.path.join(pathToExcelFile, nameOfFile + fileExtension))
+  if len(pathToExcelFile):
+    excelFile = pd.read_excel(os.path.join(pathToExcelFile, nameOfFile + fileExtension))
+  elif type(excelFileDataFrame) != str:
+    excelFile = excelFileDataFrame
+  else:
+    print("You must provide either an excel file or a video name to create a dataframe of parameters.")
+  
   if nbFramesTakenIntoAccount == -1:
     boutNbFrames = []
     boutTakenIntoAcccount = 0
@@ -77,12 +83,15 @@ def createDataFrame(dataframeOptions):
   
   # Creating labels of columns of dataframe
   # General basic information
-  basicInformation = ['Trial_ID', 'Well_ID', 'NumBout', 'BoutStart', 'BoutEnd', 'Condition', 'Genotype']
+  basicInformation = ['Trial_ID', 'Well_ID', 'NumBout', 'BoutStart', 'BoutEnd', 'Condition', 'Genotype', 'videoDuration']
   # Global parameters
   if tailAngleKinematicParameterCalculation:
-    globParam  = ['BoutDuration', 'TotalDistance', 'Speed', 'maxOfInstantaneousTBF', 'meanOfInstantaneousTBF', 'medianOfInstantaneousTBF', 'maxBendAmplitude', 'meanBendAmplitude', 'medianBendAmplitude', 'NumberOfOscillations', 'meanTBF', 'maxTailAngleAmplitude', 'deltaHead', 'firstBendTime', 'firstBendAmplitude', 'IBI', 'xmean', 'ymean']
+    globParam  = ['BoutDuration', 'TotalDistance', 'Speed', 'maxOfInstantaneousTBF', 'meanOfInstantaneousTBF', 'medianOfInstantaneousTBF', 'maxBendAmplitude', 'meanBendAmplitude', 'medianBendAmplitude', 'NumberOfOscillations', 'meanTBF', 'maxTailAngleAmplitude', 'deltaHead', 'firstBendTime', 'firstBendAmplitude', 'IBI', 'xmean', 'ymean', 'binaryClass25degMaxTailAngle']
+    if type(addToGlobalParameters) == list:
+      globParam = globParam + addToGlobalParameters
   else:
     globParam  = ['BoutDuration', 'TotalDistance', 'Speed']
+  
   # Initial raw data
   if saveRawDataInAllBoutsSuperStructure:
     if tailAngleKinematicParameterCalculation:
@@ -101,7 +110,9 @@ def createDataFrame(dataframeOptions):
   instaHeadingDiff = ['instaHeadingDiff' + str(i) for i in range(1,nbFramesTakenIntoAccount+1)]
   instaHorizDispl  = ['instaHorizDispl'  + str(i) for i in range(1,nbFramesTakenIntoAccount+1)]
   # Assembling columns
-  dfCols = basicInformation + globParam + tailAngles
+  dfCols = basicInformation + globParam
+  if computeTailAngleParamForCluster:
+    dfCols = dfCols + tailAngles
   if saveRawDataInAllBoutsSuperStructure:
     dfCols = dfCols + rawData
   if computeTailAngleParamForCluster:
@@ -119,6 +130,12 @@ def createDataFrame(dataframeOptions):
   dfParam = pd.DataFrame(columns=dfCols)
   genotypes  = []
   conditions = []
+  # This is for the potential reload from previously calculated parameters (stored in the pkl file)
+  if keepSpeedDistDurWhenLowNbBends == 1:
+    onlyKeepTheseColumns = basicInformation + ['BoutDuration', 'TotalDistance', 'Speed', 'IBI']
+  else:
+    onlyKeepTheseColumns = basicInformation
+  removeColumnsWhenAppropriate = [col for col in dfCols if not(col in onlyKeepTheseColumns)]
   # Going through each video listed in the excel file
   for videoId in range(0, len(excelFile)):
     if excelFile.loc[videoId, 'path'] == "defaultZZoutputFolder":
@@ -137,13 +154,34 @@ def createDataFrame(dataframeOptions):
     include   = excelFile.loc[videoId, 'include']
     include = eval('[' + include + ']')
     include = include[0]
-    
-    with open(os.path.join(path, 'results_' + trial_id + '.txt')) as f:
-      supstruct = json.load(f)
+    if (not(os.path.exists(os.path.join(path, trial_id + '.pkl'))) or forcePandasDfRecreation):
+      with open(os.path.join(path, 'results_' + trial_id + '.txt')) as f:
+        supstruct = json.load(f)
+        firstFrame = supstruct["firstFrame"]
+        lastFrame  = supstruct["lastFrame"]
+    else:
+      print("reloading previously calculated parameters")
+      dfReloadedVid = pd.read_pickle(os.path.join(path, trial_id + '.pkl'))
+      for idx, cond in enumerate(condition):
+        indForWellId = (dfReloadedVid['Well_ID'] == idx)
+        if include[idx]:
+          dfReloadedVid.loc[indForWellId, 'Condition'] = cond
+          dfReloadedVid.loc[indForWellId, 'Genotype']  = genotype[idx]
+          if minNbBendForBoutDetect > 0:
+            ind           = (dfReloadedVid['NumberOfOscillations'] < minNbBendForBoutDetect/2)
+            dfReloadedVid.loc[ind, removeColumnsWhenAppropriate] = float('NaN')
+          if not(genotype[idx] in genotypes):
+            genotypes.append(genotype[idx])
+          if not(condition[idx] in conditions):
+            conditions.append(condition[idx])
+        else:
+          dfReloadedVid.drop([idx2 for idx2, belongsToWell in enumerate(indForWellId) if belongsToWell])
+      dfParam = pd.concat([dfParam, dfReloadedVid])
     
     # Going through each well of the video
     for Well_ID, Cond in enumerate(condition):
-      if include[Well_ID]:
+      # Not going through this loop if we've already reloaded parameters
+      if include[Well_ID] and (not(os.path.exists(os.path.join(path, trial_id + '.pkl'))) or forcePandasDfRecreation):
         print("trial_id:", trial_id, " ; Well_ID:", Well_ID)
         dfParamForWell = pd.DataFrame(columns=dfCols)
         curBoutId = 0
@@ -158,7 +196,7 @@ def createDataFrame(dataframeOptions):
                 # Initial basic information
                 
                 toPutInDataFrameColumn = basicInformation
-                toPutInDataFrame       = [trial_id, Well_ID, NumBout, dataForBout['BoutStart'], dataForBout['BoutEnd'], condition[Well_ID], genotype[Well_ID]]
+                toPutInDataFrame       = [trial_id, Well_ID, NumBout, dataForBout['BoutStart'], dataForBout['BoutEnd'], condition[Well_ID], genotype[Well_ID], (lastFrame - firstFrame) / fq]
                 
                 if not(genotype[Well_ID] in genotypes):
                   genotypes.append(genotype[Well_ID])
@@ -168,7 +206,7 @@ def createDataFrame(dataframeOptions):
                 # Calculates the global kinematic parameters and stores them the dataframe
                 
                 previousBoutEnd = supstruct["wellPoissMouv"][Well_ID][fishId][NumBout-1]["BoutEnd"] if NumBout > 0 else 0
-                listOfGlobalParameters = getGlobalParameters(dataForBout, fq, pixelsize, frameStepForDistanceCalculation, previousBoutEnd, globParam)
+                listOfGlobalParameters = getGlobalParameters(dataForBout, fq, pixelsize, frameStepForDistanceCalculation, previousBoutEnd, globParam, firstFrame, lastFrame)
                 
                 toPutInDataFrameColumn = toPutInDataFrameColumn + globParam
                 toPutInDataFrame       = toPutInDataFrame       + listOfGlobalParameters
@@ -222,44 +260,50 @@ def createDataFrame(dataframeOptions):
               
               else:
                 
-                if keepSpeedDistDurWhenLowNbBends:
+                # Initial basic information
                 
-                  # Initial basic information
-                  
-                  toPutInDataFrameColumn = basicInformation
-                  toPutInDataFrame       = [trial_id, Well_ID, NumBout, dataForBout['BoutStart'], dataForBout['BoutEnd'], condition[Well_ID], genotype[Well_ID]]
-                  
-                  if not(genotype[Well_ID] in genotypes):
-                    genotypes.append(genotype[Well_ID])
-                  if not(condition[Well_ID] in conditions):
-                    conditions.append(condition[Well_ID])
+                toPutInDataFrameColumn = basicInformation
+                toPutInDataFrame       = [trial_id, Well_ID, NumBout, dataForBout['BoutStart'], dataForBout['BoutEnd'], condition[Well_ID], genotype[Well_ID], (lastFrame - firstFrame) / fq]
+                
+                if not(genotype[Well_ID] in genotypes):
+                  genotypes.append(genotype[Well_ID])
+                if not(condition[Well_ID] in conditions):
+                  conditions.append(condition[Well_ID])
+                
+                if keepSpeedDistDurWhenLowNbBends:
                   
                   # Calculating the global kinematic parameters and more and stores them the dataframe
                   
                   previousBoutEnd = supstruct["wellPoissMouv"][Well_ID][fishId][NumBout-1]["BoutEnd"] if NumBout > 0 else 0
-                  listOfGlobalParameters = getGlobalParameters(dataForBout, fq, pixelsize, frameStepForDistanceCalculation, previousBoutEnd, ['BoutDuration', 'TotalDistance', 'Speed', 'IBI'])
+                  listOfGlobalParameters = getGlobalParameters(dataForBout, fq, pixelsize, frameStepForDistanceCalculation, previousBoutEnd, ['BoutDuration', 'TotalDistance', 'Speed', 'IBI'], firstFrame, lastFrame)
                   
                   toPutInDataFrameColumn = toPutInDataFrameColumn + ['BoutDuration', 'TotalDistance', 'Speed', 'IBI']
                   toPutInDataFrame       = toPutInDataFrame       + listOfGlobalParameters
                   
-                  # Adding bout parameters to the dataframe created for the current well
-                  
-                  dfParamForWell.loc[curBoutId, toPutInDataFrameColumn] = toPutInDataFrame
-                  curBoutId = curBoutId + 1
-        
+                # Adding bout parameters to the dataframe created for the current well
+                
+                dfParamForWell.loc[curBoutId, toPutInDataFrameColumn] = toPutInDataFrame
+                curBoutId = curBoutId + 1
+      
         # Adding dataframe created for the current frame to the dataframe for the whole set of videos
         dfParam = pd.concat([dfParam, dfParamForWell])
   
-  # Saving the dataframe
+  # Saving the dataframe  
+  if 'level_0' in dfParam.columns:
+    dfParam = dfParam.drop(['level_0'], axis=1)
   dfParam = dfParam.reset_index()
   
   # Saving dataframe for the whole set of videos as a pickle file
-  outfile = open(os.path.join(resFolder, nameOfFile), 'wb')
+  outfile = open(os.path.join(resFolder, nameOfFile + '.pkl'), 'wb')
   pickle.dump(dfParam,outfile)
   outfile.close()
   
+  file1 = open(os.path.join(resFolder, 'frameStepForDistanceCalculationUsed.json'), "w")
+  file1.write(str(frameStepForDistanceCalculation))
+  file1.close()
+  
   # Saving dataframe for the whole set of videos as a matlab file
   if saveAllBoutsSuperStructuresInMatlabFormat:
-    scipy.io.savemat(os.path.join(resFolder, nameOfFile + '.mat'), dfParam)
+    scipy.io.savemat(os.path.join(resFolder, nameOfFile + '.mat'), {'struct1':dfParam.to_dict("list")})
     
   return [conditions, genotypes, nbFramesTakenIntoAccount, globParam]
