@@ -18,6 +18,13 @@ import time
 import cv2
 import zebrazoom.videoFormatConversion.zzVideoReading as zzVideoReading
 import json
+import subprocess
+import glob
+
+try:
+  from PyQt6.QtWidgets import QApplication
+except ImportError:
+  from PyQt5.QtWidgets import QApplication
 
 from zebrazoom.code.vars import getGlobalVariables
 globalVariables = getGlobalVariables()
@@ -119,18 +126,32 @@ def mainZZ(pathToVideo, videoName, videoExt, configFile, argv):
     # Creating output folder
     if not(hyperparameters["reloadWellPositions"]) and not(hyperparameters["reloadBackground"]) and not(os.path.exists(os.path.join(outputFolderVideo, 'intermediaryWellPositionReloadNoMatterWhat.txt'))) and not(hyperparameters["dontDeleteOutputFolderIfAlreadyExist"]):
       if os.path.exists(outputFolderVideo):
-        shutil.rmtree(outputFolderVideo)
-      while True:
+        if glob.glob(os.path.join(outputFolderVideo, 'results_*.txt')) and glob.glob(os.path.join(outputFolderVideo, '*.avi')):
+          pastNumbersTaken = 1
+          while os.path.exists(outputFolderVideo + '_PastTracking_' + str(pastNumbersTaken)) and pastNumbersTaken < 99:
+            pastNumbersTaken += 1
+          shutil.move(outputFolderVideo, outputFolderVideo + '_PastTracking_' + str(pastNumbersTaken))
+        else:
+          shutil.rmtree(outputFolderVideo)
+      if hyperparameters["tryCreatingFolderUntilSuccess"]:
+        while True:
+          try:
+            os.mkdir(outputFolderVideo)
+            break
+          except OSError as e:
+            print("waiting inside except")
+            time.sleep(0.1)
+          else:
+            print("waiting")
+            time.sleep(0.1)
+      else:
         try:
           os.mkdir(outputFolderVideo)
-          break
         except OSError as e:
-          print("waiting inside except")
           time.sleep(0.1)
         else:
-          print("waiting")
           time.sleep(0.1)
-    
+  
   # Saving the configuration file used
   with open(os.path.join(outputFolderVideo, 'configUsed.json'), 'w') as outfile:
     json.dump(configFile, outfile)
@@ -259,6 +280,14 @@ def mainZZ(pathToVideo, videoName, videoExt, configFile, argv):
     # Various post-processing options depending on configuration file choices
     dataPostProcessing(outputFolderVideo, superStruct, hyperparameters, videoName, videoExt)
   
+  try:
+    with open(os.path.join(os.path.join(hyperparameters["outputFolder"], hyperparameters["videoName"]), 'ZebraZoomVersionUsed.txt'), 'w') as fp:
+      subprocess.run(["pip", "show",  "zebrazoom"], stdout=fp)
+  except:
+    fileVersionUsed = open(os.path.join(os.path.join(hyperparameters["outputFolder"], hyperparameters["videoName"]), 'ZebraZoomVersionUsed.txt'), 'w')
+    fileVersionUsed.write("Was not able to retrive the version number used.")
+    fileVersionUsed.close()
+  
   # Copying output result folder in another folder
   if len(hyperparameters["additionalOutputFolder"]):
     if os.path.isdir(hyperparameters["additionalOutputFolder"]):
